/*************************************************************************
* Copyright (c) 2004 Altera Corporation, San Jose, California, USA.      *
* All rights reserved. All use of this software and documentation is     *
* subject to the License Agreement located at the end of this file below.*
**************************************************************************
* Description:                                                           *
* The following is a simple hello world program running MicroC/OS-II.The *
* purpose of the design is to be a very simple application that just     *
* demonstrates MicroC/OS-II running on NIOS II.The design doesn't account*
* for issues such as checking system call return codes. etc.             *
*                                                                        *
* Requirements:                                                          *
*   -Supported Example Hardware Platforms                                *
*     Standard                                                           *
*     Full Featured                                                      *
*     Low Cost                                                           *
*   -Supported Development Boards                                        *
*     Nios II Development Board, Stratix II Edition                      *
*     Nios Development Board, Stratix Professional Edition               *
*     Nios Development Board, Stratix Edition                            *
*     Nios Development Board, Cyclone Edition                            *
*   -System Library Settings                                             *
*     RTOS Type - MicroC/OS-II                                           *
*     Periodic System Timer                                              *
*   -Know Issues                                                         *
*     If this design is run on the ISS, terminal output will take several*
*     minutes per iteration.                                             *
**************************************************************************/


#include <stdio.h>
#include <string.h>
#include "includes.h"
#include "alt_types.h"
#include <sys/alt_irq.h>
#include "altera_up_avalon_character_lcd.h"
#include "altera_up_avalon_parallel_port.h"


/* Definition of Task Stacks */
#define   TASK_STACKSIZE       2048
OS_STK    task_green1_stk[TASK_STACKSIZE];
OS_STK    task_green2_stk[TASK_STACKSIZE];
OS_STK    task_red1_stk[TASK_STACKSIZE];

/* Definition of Task Priorities */

#define TASK_RED1_PRIORITY      5
#define MUTEX_PRIORITY			6
#define TASK_GREEN1_PRIORITY    7
#define TASK_GREEN2_PRIORITY    8
#define TASK_DISPLAY 			9
#define TIME_COUNTER            10
#define queue_SIZE             15


volatile int edge_capture;
int const STOP_TIME = 5;
int count;
int greenLight = 0; //0 - 1 swiatlo, 1 - 2 swiatlo
int state = 0; // 0 - normalny, 1 awaryjny
void* message_queue[queue_SIZE];


OS_EVENT *mutex;
OS_EVENT *queue;
INT8U interruptionCount;
INT8U partition_size[10][50];

alt_up_parallel_port_dev* switches;
alt_up_parallel_port_dev* ledr;
alt_up_parallel_port_dev* ledg;
alt_up_character_lcd_dev* lcd;
alt_up_parallel_port_dev* button;
alt_up_parallel_port_dev* seg30;


void resetClock(){
	count = 10;
}


unsigned char decodeDigit(unsigned char s){
	switch(s)
	{
	case 0: return 63;
	case 1: return 6;
	case 2: return 91;
	case 3: return 79;
	case 4: return 102;
	case 5: return 109;
	case 6: return 125;
	case 7: return 7;
	case 8: return 127;
	case 9: return 111;
	default: return 0;
	}
}

void task_green1(void* pdata)
{
  unsigned int switchesData;
  unsigned int timer = TIME_COUNTER;
  INT8U car;
  INT8U blinking=0;
  while (1)
  {
	  puts("green1");
	INT8U err;
	OSMutexPend(mutex,0,&err);
	switchesData = alt_up_parallel_port_read_data(switches);

	if((switchesData&2) == 2)
		car = 1;
	else
	{
		car = 0;
	}

	if(state!=1 && greenLight == 0){
		if(car == 1 && timer == 0){
			greenLight = 1;
			timer = TIME_COUNTER;
		}
		if(greenLight == 0){
			alt_up_parallel_port_write_data(ledr, 1);
			alt_up_parallel_port_write_data(ledg, 2);

			alt_up_parallel_port_write_data(seg30, decodeDigit(timer));
			if(car == 0 && timer == 5 && blinking == 1)
			{
				alt_up_parallel_port_write_data(seg30, 0);
			}

			if((car == 0 && timer > STOP_TIME) || car==1) {
				timer--;
				}
		}
	}
	//OSMutexPost(mutex);
	blinking = (blinking + 1) % 2;
	OSTimeDlyHMSM(0,0,1,0);
	OSMutexPost(mutex);
  }
}

void task_green2(void* pdata)
{
  unsigned int switchesData;
  unsigned int timer = TIME_COUNTER;
  INT8U car;
  INT8U blinking=0;
  while (1)
  {
	  puts("green2");
	INT8U err;
	OSMutexPend(mutex,0,&err);
	switchesData = alt_up_parallel_port_read_data(switches);
	if((switchesData&1) == 1)
		car = 1;
	else
	{
		car = 0;
	}

	if(state!=1 && greenLight == 1){
		if(car == 1 && timer == 0){
			greenLight = 0;
			timer = TIME_COUNTER;
		}
		if(greenLight == 1){
			alt_up_parallel_port_write_data(ledr, 2);
		    alt_up_parallel_port_write_data(ledg, 1);

			alt_up_parallel_port_write_data(seg30, decodeDigit(timer));
			if(car == 0 && timer == 5 && blinking == 1)
			{
				alt_up_parallel_port_write_data(seg30, 0);
			}

			if((car == 0 && timer > STOP_TIME) || car==1)
			{
			    --timer;
			}
		}
	}
	//OSMutexPost(mutex);
	blinking = (++blinking ) % 2;
	OSTimeDlyHMSM(0,0,1,0);
	OSMutexPost(mutex);
  }
}

void setEmergencyState(){
	puts("emergency start");
	INT8U err;
	OSMutexPend(mutex,1000,&err);
	state = 1;
	alt_up_parallel_port_write_data(ledr, 3);
	alt_up_character_lcd_set_cursor_pos(lcd,0,0);
	alt_up_character_lcd_string(lcd,"Stan awaryjny!");
	OSMutexPost(mutex);
}

void setNormalState(){
	puts("emergency stop");
	INT8U err;
	OSMutexPend(mutex,1000,&err);
	state =  0;
	alt_up_character_lcd_set_cursor_pos(lcd,0,0);
	alt_up_character_lcd_string(lcd,"Stan normalny!");
	OSMutexPost(mutex);
}
void task_red1(void *pdata){
	INT8U count = 0;
	INT8U err;

	while(1){
		puts("red");
		void* temp;
		temp = OSQAccept(queue,&err);
		if(temp!=NULL)
		{
			count = (++count)%2;

		if(count==1)
		{
			setEmergencyState();
		}
		else
		{
			setNormalState();
		}
		}
		OSTimeDlyHMSM(0,0,0,500);
	}
}
void interrupt_handler(void* context, alt_u32 id) {
	INT8U message= interruptionCount;
	OSQPost(queue,(void *)&message);
	interruptionCount = (++interruptionCount) % 2;
	alt_up_parallel_port_clear_edge_capture(button);
}

/* The main function creates two task and starts multi-tasking */
int main(void)
{
	INT8U err;
	switches = alt_up_parallel_port_open_dev(SWITCHES_NAME);
    lcd = alt_up_character_lcd_open_dev(LCD_NAME);
	ledr = alt_up_parallel_port_open_dev(LEDR_NAME);
	ledg = alt_up_parallel_port_open_dev(LEDG_NAME);
	seg30 = alt_up_parallel_port_open_dev(SEG30_NAME);
    button = alt_up_parallel_port_open_dev(KEYS_NAME);

    alt_up_character_lcd_init (lcd);
    alt_irq_register(KEYS_IRQ, (void*) &edge_capture, interrupt_handler);
    alt_irq_enable(KEYS_IRQ);
    alt_up_parallel_port_set_interrupt_mask(button, 4);
	greenLight =0;
	interruptionCount = 0;
    OSInit();

    mutex = OSMutexCreate(MUTEX_PRIORITY,&err);
	queue = OSQCreate(&message_queue[0],queue_SIZE);


    OSTaskCreateExt(task_green1,
                  NULL,
                  (void *)&task_green1_stk[TASK_STACKSIZE-1],
                  TASK_GREEN1_PRIORITY,
                  TASK_GREEN1_PRIORITY,
                  task_green1_stk,
                  TASK_STACKSIZE,
                  NULL,
                  0);

    OSTaskCreateExt(task_green2,
                  NULL,
                  (void *)&task_green2_stk[TASK_STACKSIZE-1],
                  TASK_GREEN2_PRIORITY,
                  TASK_GREEN2_PRIORITY,
                  task_green2_stk,
                  TASK_STACKSIZE,
                  NULL,
                  0);

    OSTaskCreateExt(task_red1,
                  NULL,
                  (void *)&task_red1_stk[TASK_STACKSIZE-1],
                  TASK_RED1_PRIORITY,
                  TASK_RED1_PRIORITY,
                  task_red1_stk,
                  TASK_STACKSIZE,
                  NULL,
                  0);

    puts("start");
  OSStart();

  return 0;
}
