/*************************************************************************
* Copyright (c) 2004 Altera Corporation, San Jose, California, USA.      *
* All rights reserved. All use of this software and documentation is     *
* subject to the License Agreement located at the end of this file below.*
**************************************************************************
* Description:                                                           *
* The following is a simple hello world program running MicroC/OS-II.The *
* purpose of the design is to be a very simple application that just     *
* demonstrates MicroC/OS-II running on NIOS II.The design doesn't account*
* for issues such as checking system call return codes. etc.             *
*                                                                        *
* Requirements:                                                          *
*   -Supported Example Hardware Platforms                                *
*     Standard                                                           *
*     Full Featured                                                      *
*     Low Cost                                                           *
*   -Supported Development Boards                                        *
*     Nios II Development Board, Stratix II Edition                      *
*     Nios Development Board, Stratix Professional Edition               *
*     Nios Development Board, Stratix Edition                            *
*     Nios Development Board, Cyclone Edition                            *
*   -System Library Settings                                             *
*     RTOS Type - MicroC/OS-II                                           *
*     Periodic System Timer                                              *
*   -Know Issues                                                         *
*     If this design is run on the ISS, terminal output will take several*
*     minutes per iteration.                                             *
**************************************************************************/


#include <stdio.h>
#include "includes.h"
#include "alt_types.h"
#include <sys/alt_irq.h>
#include "altera_up_avalon_parallel_port.h"
#include "altera_up_avalon_character_lcd.h"
#include <string.h>

/* Definition of Task Stacks */
#define   TASK_STACKSIZE       2048
OS_STK    task1_stk[TASK_STACKSIZE];
OS_STK    task2_stk[TASK_STACKSIZE];
OS_STK    task3_stk[TASK_STACKSIZE];

/* Definition of Task Priorities */

#define TASK1_PRIORITY      6
#define TASK2_PRIORITY      7
#define TASK3_PRIORITY      8

/* Prints "Hello World" and sleeps for three seconds */
void task1(void* pdata)
{
  while (1)
  {
    printf("Hello from task1\n");
    OSTimeDlyHMSM(0, 0, 3, 0);
  }
}
/* Prints "Hello World" and sleeps for three seconds */
void task2(void* pdata)
{
  while (1)
  {
    printf("Hello from task2\n");
    OSTimeDlyHMSM(0, 0, 3, 0);
  }
}
OS_MEM *partycja;
INT8U rozmiar_partycji[10][50];
OS_EVENT *myMailBox;



void task3(void* pdata)
{
          char * tmp;
          INT8U err;

  while (1)
  {


          tmp = (char*) OSMboxPend(myMailBox, 0, &err);

          printf("Task3: %s\n", tmp);

        if(OSMemPut(partycja, tmp) != OS_NO_ERR)
        {
            puts("ERROR MEMPUT");
        }
  }
}


int value = 0;
volatile int edge_capture;
alt_up_character_lcd_dev * lcd;
alt_up_parallel_port_dev* buttons;

void interrupt_handler(void* context, alt_u32 id) {
    value++;
    char tmp[15], *blok;
    INT8U err;
    sprintf(tmp, "Value: %2d", value);
    alt_up_character_lcd_set_cursor_pos(lcd, 0, 0);
    alt_up_character_lcd_string(lcd, tmp);
    alt_up_parallel_port_clear_edge_capture(buttons);



    blok = (char * ) OSMemGet(partycja, &err);
    if(blok == NULL)
    {
        //puts("ERROR");
    alt_up_character_lcd_set_cursor_pos(lcd, 0, 1);
    alt_up_character_lcd_string(lcd, "blok ERROR");
    }
    strcpy(blok, tmp);
    OSMboxPost(myMailBox, blok);

}


/* The main function creates two task and starts multi-tasking */
int main(void)
{
    INT8U err;

    lcd = alt_up_character_lcd_open_dev(LCD_NAME);
    alt_up_character_lcd_init (lcd);
    buttons = alt_up_parallel_port_open_dev(KEYS_NAME);
    alt_irq_register(KEYS_IRQ, (void*) &edge_capture, interrupt_handler);
    alt_irq_enable(KEYS_IRQ);
    alt_up_parallel_port_set_interrupt_mask(buttons, 4);

    OSInit();



    partycja = OSMemCreate(rozmiar_partycji, 10, 50, &err);
    if (partycja == NULL)
    {
        puts("ERROR PARTITION1");
    }

    myMailBox = OSMboxCreate(NULL);





  OSTaskCreateExt(task1,
                  NULL,
                  (void *)&task1_stk[TASK_STACKSIZE-1],
                  TASK1_PRIORITY,
                  TASK1_PRIORITY,
                  task1_stk,
                  TASK_STACKSIZE,
                  NULL,
                  0);
              
               
  OSTaskCreateExt(task2,
                  NULL,
                  (void *)&task2_stk[TASK_STACKSIZE-1],
                  TASK2_PRIORITY,
                  TASK2_PRIORITY,
                  task2_stk,
                  TASK_STACKSIZE,
                  NULL,
                  0);

  OSTaskCreateExt(task3,
                  NULL,
                  (void *)&task3_stk[TASK_STACKSIZE-1],
                  TASK3_PRIORITY,
                  TASK3_PRIORITY,
                  task3_stk,
                  TASK_STACKSIZE,
                  NULL,
                  0);

  OSStart();
  return 0;
}

/******************************************************************************
*                                                                             *
* License Agreement                                                           *
*                                                                             *
* Copyright (c) 2004 Altera Corporation, San Jose, California, USA.           *
* All rights reserved.                                                        *
*                                                                             *
* Permission is hereby granted, free of charge, to any person obtaining a     *
* copy of this software and associated documentation files (the "Software"),  *
* to deal in the Software without restriction, including without limitation   *
* the rights to use, copy, modify, merge, publish, distribute, sublicense,    *
* and/or sell copies of the Software, and to permit persons to whom the       *
* Software is furnished to do so, subject to the following conditions:        *
*                                                                             *
* The above copyright notice and this permission notice shall be included in  *
* all copies or substantial portions of the Software.                         *
*                                                                             *
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR  *
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,    *
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE *
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER      *
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING     *
* FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER         *
* DEALINGS IN THE SOFTWARE.                                                   *
*                                                                             *
* This agreement shall be governed in all respects by the laws of the State   *
* of California and by the laws of the United States of America.              *
* Altera does not recommend, suggest or require that this reference design    *
* file be used in conjunction or combination with any other product.          *
******************************************************************************/
